/**
 * Klassen inneholder metoder som henter nsket element 
 * i XML-dokument ved hjelp av XPath. Returverdiene til 
 * metodene varierer. Metoden XPath ligger i klassen Parsing.
 * 
 * @author Trude
 */
package filmweb;

import java.io.IOException;
import java.util.Vector;

import javax.xml.xpath.XPathExpressionException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class ComponentList {
	/**
	 * Deklarering av variabler:
	 */
	Console console;
	Parsing parsing = null;
	
	//-------------------------------------------------------------------------------------------------------
	/**
	 * Konstruktr av klassen Console.
	 * 
	 * @param console
	 */
	public ComponentList(Console c) {
		console = c;
		parsing = new Parsing(null);
	}

	//-------------------------------------------------------------------------------------------------------
	/**
	 * Metoden getTitle henter overskriften for GUI. Overskriften ligger
	 * som Descriptor i DIM i XML-dokument. Metoden tar utgangspunkt
	 * i MethodInfo da alle DIM inneholder denne tag.
	 * 
	 * @param Document didDocument, parset XML-dokument ligger lagret her
	 * @return String tekststreng med overksirften
	 */
	public String getTitle(Document didDocument){
		try {
			NodeList list = parsing.doXPath(didDocument, "//dip:MethodInfo");
		
			for(int i = 0; i < list.getLength(); i++) {
				Element element = (Element)list.item(i);
				Element el = (Element)element.getParentNode().getParentNode().getParentNode().getParentNode();
				String statementText = getStatementText(el);
				return statementText;
			}

		} catch (XPathExpressionException e) {
			e.printStackTrace();
		}
		return null;
	}
	//----------------------------------------------------------------------------------------------------------
	/**
	 * Metoden getStatementText henter teksten i Statement. Metoden er generell
	 * og kan benyttes til  hente teskt i hvilken som helst Statement.
	 *  
	 * @param Element element hvor Statement er child til
	 * @return String som inneholder teksten i Statement
	 */
	public String getStatementText(Element element){
		try {
			NodeList statementList = parsing.doXPath(element, "did:Descriptor//did:Statement");
			
			for(int i = 0; i < statementList.getLength(); i++){
				Element statementEl = (Element)statementList.item(i);
				String text = statementEl.getTextContent();
				return text;
			}
		} catch (XPathExpressionException e) {
			e.printStackTrace();
		}
		return null;
	}
	//---------------------------------------------------------------------------------------------
	/**
	 * Metoden getComponentList lager en liste over alle Component som 
	 * inneholder MethodInfo. Dette blir da listen over alle DIM i XML-
	 * dokumentet.
	 * 
	 * @param Document didDocument, parset XML-dokument ligger lagret her
	 * @return Vector med liste av DIM.
	 */
	public Vector getComponentList(Document didDocument) throws Exception{
		Vector componentList = new Vector();
		
		try {
			NodeList list = parsing.doXPath(didDocument, "//dip:MethodInfo");
		
			for(int i = 0; i < list.getLength(); i++) {
				Element element = (Element)list.item(i);
				componentList.addElement( element.getParentNode().getParentNode().getParentNode());
			}

		} catch (XPathExpressionException e) {
			e.printStackTrace();
		}
		return componentList;
	}
	
	//---------------------------------------------------------------------------------------------
	/**
	 * Metoden getListOfElement henter listen fra getMenuList og returnerer i
	 * vektoren list.
	 * 
	 * @param Vector componentList vektor med liste av Component
	 * @return Vector med menyliste
	 * @throws Exception
	 */
	
	public Vector getListOfElement(Vector componentList) throws Exception{
		Vector menuList;
		Vector list = new Vector();
		for(int j= 0; j < componentList.size(); j++){
			Element element = (Element)componentList.elementAt(j);
			menuList = getMenuList(element);
			list.addElement(menuList);
		}
		return list;
		
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden getMenuList henter alle Resource og Statement i Descriptor i xml-
	 * dokumentet. Metoden henter videre ut mimeType, ref og plain tekst og legger 
	 * i vektoren menuList.
	 * 
	 * @param Element element, funnet i metoden getListOfElement
	 * @return Vector med liste av alle ressurser og statement i Descriptor.
	 * @throws XPathExpressionException
	 * @throws IOException
	 */
	public Vector getMenuList(Element element) throws XPathExpressionException, IOException{
		NodeList list = parsing.doXPath(element, "did:Descriptor//did:Resource " +
												 "| did:Descriptor//did:Statement");
		Vector menuList = new Vector();
		
		for(int i = 0; i < list.getLength(); i++){
			
			Element type = (Element)list.item(i);
			String mimeType = type.getAttribute("mimeType");
			if(mimeType == ""){mimeType = null;}
			String ref = type.getAttribute("ref");
			if(ref == ""){ref = null;}
			String text = type.getTextContent();
			if(text == ""){text = null;}
			menuList.addElement(new Object[]{mimeType, ref, text});
			
		}
		return menuList;
	}
	//----------------------------------------------------------------------------------------------
	/**
	 * Metoden getDIMResourceList henter ressursene i xml-dokumentet som 
	 * inneholder ECMAScriptkode. Sjekker dette ved  benytte mimeType til
	 * Reource.
	 * 
	 *  @param Element component, component med Resource 
	 *  @return Vector, vektor med alle Resource med ECMAScriptkode
	 *  @throws XPathExpressionException
	 */
	public Vector getDIMResourceList(Element component) throws XPathExpressionException{
		
		Vector resourceList = new Vector();
		
		NodeList list = parsing.doXPath(component, "did:Resource");
		
		for(int i = 0; i < list.getLength(); i++) {
			Element resourceElement = (Element)list.item(i);
			
			String mimeType = resourceElement.getAttribute("mimeType");
			if(mimeType.equals("application/mp21-method")){
				resourceList.addElement(resourceElement);
				
			}
			
		}
		
		return resourceList;
	}
	//-----------------------------------------------------------------------------------------------
	/**
	 * Metoden getFilmwebIcon henter referansen til ikonet til 
	 * filmweb i xml-dokumentet.
	 * @param Document didDocument, parset XML-dokument ligger lagret her 
	 * @return String, tekststreng med referanse til filmwebikon
	 */
	public String getFilmwebIcon(Document didDocument) throws Exception{
		
		NodeList iconResource = parsing.doXPath(didDocument, 
				"//did:Container//did:Descriptor//did:Component//did:Resource");
		
		for(int i = 0; i < iconResource.getLength(); i++) {
			Element resourceElement = (Element)iconResource.item(i);
			
			String iconFilmweb = resourceElement.getAttribute("ref");
			return iconFilmweb;
		}
		return null;
	}

}
