/**
 * Klassen Console er startklassen for programmet. Den inneholder main som 
 * starter hele programmet, main inneholder stien for hvor XML-dokumentet
 * er lagret.
 * 
 * Klassen inneholder ogs initieringsmedtodene for 
 * ecmascriptmotoren og grafikken. 
 * 
 * @author Trude
 */
package filmweb;

import grafikk.MainJFrame;
import grafikk.TextInput;

import java.util.Vector;

import mpeg21.DIP;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class Console implements ToButtonInterface{
	
	/**
	 * Deklarering av variabler:
	 */
	TextInput textInput;
	MainJFrame frame = null;
	ComponentList componentList;
	Parsing parsing = null;
	
	Document didDocument;
	Context cx ;
	Scriptable scope;
	Element comp;
	Vector compList;
	
	//-------------------------------------------------------------------------------------
	/**
	 * Konstruktren av klassen Console.
	 */
	public Console() {
		frame = new MainJFrame(this);
		componentList = new ComponentList(this);
		textInput = new TextInput(this);
		parsing = new Parsing(this);
	}
	//-------------------------------------------------------------------------------------
	public MainJFrame getFrame() {
		return frame;
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Her er main som starter programmet. 
	 * 
	 * Main inneholder stien for hvor
	 * XML-dokumentet er lagret. Denne m endres fr kjring av programmet.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		
		Console console = new Console();
		try {
			console.initDigitalItemProcessing("C:\\Documents and Settings\\Trude\\workspace\\testprogram.xml");
			console.initGraphics();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	//--------------------------------------------------------------------------------
    /**
     * Metoden executeFunction tolker ECMAScript-koden i xml-dokumentet
     * ved hjelp av en ECMAScriptmotor.
     * 
     * For  kunne bruke ECMAScriptmotoren m prosjektfilene
     * Rhino og Xerces2 importeres.
     * 
     * @param scriptResource Element med ECMAScriptkode.
     */
	public void executeFunction(Element scriptResource) {

		String script = scriptResource.getTextContent();
		
		Context cx = Context.enter();
		
		//System.out.println("Attempting to execute function:\n" + script);
		
		Function f = cx.compileFunction(scope, script, "<cmd>", 1, null);

        Object result = f.call(cx, scope, f, new Object[]{new Integer(10)});			

        //Convert the result to a string and print it.
        System.err.println("Result: " + cx.toString(result));	
        
        close();
	}
	//---------------------------------------------------------------------------------
	/**
	 * Metoden init parser xml-dokumentet og lagrer det i variabelen
	 * didDocument. Metodene initierer ECMAScriptmotor.
	 * 
	 * @param xmlString stien for hvor XML-dokumentet er lagret.
	 * @throws Exception
	 */
	public void initDigitalItemProcessing(String xmlString) throws Exception{
		didDocument = parsing.load(xmlString);
		
		cx = Context.enter();
        scope = cx.initStandardObjects();   
        
        Object document = Context.javaToJS(didDocument, scope);
        ScriptableObject.putProperty(scope, "didDocument", document);
        
        
        DIP dip = new DIP(didDocument);
        Object jsDip = Context.javaToJS(dip, scope);
        ScriptableObject.putProperty(scope, "DIP", jsDip);
                
	}
	//---------------------------------------------------------------------------------------
	/**
	 * Metoden initGraphics inititiere grafikken til start-GUI.
	 * 
	 * @throws Exception
	 */
	public void initGraphics() throws Exception {
		String iconPath = componentList.getFilmwebIcon(didDocument);
		
		MainJFrame frame = getFrame();
		
		frame.setHeader("<html><img src =" + iconPath+ "></img></html>", "text/html");
		
		compList = componentList.getComponentList(didDocument);
		Vector w = componentList.getListOfElement(compList);

		String overskrift = componentList.getTitle(didDocument);
		frame.openList(w, overskrift, "text/html");		
	}
	//---------------------------------------------------------------------------------
	/**
	 * Metoden leftButton avslutter GUI.
	 */
	public void leftButton(){
		
		System.exit(0);
	}
	
	//---------------------------------------------------------------------------------
	/**
	 * Metoden rigtButton henter informasjon fra metoden run i MainJFrame
	 * nr knappen velg er trykket. En index fra run forteller hva som er 
	 * valgt p menylisten. Metoden kjrer s programmet videre med det 
	 * valget som er gjort i menylisten.   
	 */
	public void rightButton() {
		
		int index = frame.list.getSelectedIndex();
		//System.out.println(index);
		
		Vector resourceVector;

		if(index == 5){
			javax.swing.SwingUtilities.invokeLater(new Runnable() {
	            public void run() {
	                textInput.createAndShowGUI();
	            }
	        });
		}
		else if(index != -1){
			try {
				Element chosenComponent = (Element)compList.elementAt(index);
				resourceVector = componentList.getDIMResourceList(chosenComponent); 
				if(resourceVector.size()>0){
				
				Element resourceElement = (Element)resourceVector.elementAt(0);
				System.out.println(resourceElement);
				
				executeFunction(resourceElement);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	
	//---------------------------------------------------------------------------------
	/**
	 * Metoden close avslutter programmet.
	 */
	public void close() {
		Context.exit();
	}
	
}
