/**
 * Metoden sjekker namespaceinnholdet referert i klassen Parsing.
 * Namespace ligger verst i XML-dokument.
 * 
 * @author Thomas Skjlberg
 */
package filmweb;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.NamespaceContext;

public class NamespaceContextImpl implements NamespaceContext {
	/**
	 * Deklarering av variabler:
	 */
	protected Map map = new HashMap();
	//-------------------------------------------------------------------------------------------------------------------------------------
	/**
	 * Metoden setter namespace.
	 * 
	 * @param prefix
	 * @param namespaceURI
	 */
	public void setNamespace(String prefix, String namespaceURI) {
		map.put(prefix, namespaceURI);
	}
	//-------------------------------------------------------------------------------------------------------------------------------------
	/**
	 * Metoden henter namespace.
	 */
	public String getNamespaceURI(String prefix) {
		if(prefix == null) throw new IllegalArgumentException("null prefix in " + getClass().getName() + ".getNamespaceURI(String prefix)");
		return (String) map.get(prefix);

	}
	//-------------------------------------------------------------------------------------------------------------------------------------
	/**
	 * Metoden namespaceURI henter URI til namespace.
	 * 
	 * @param String namespaceURI
	 * @return String
	 */
	public String getPrefix(String namespaceURI) {

		Set keys = map.keySet();

		for (Iterator iterator = keys.iterator(); iterator.hasNext();)
		{
			String prefix = (String) iterator.next();
			String uri = (String) map.get(prefix);

			if (uri.equals(namespaceURI))
				return prefix;
		}
		return null;

	}
	//------------------------------------------------------------------------------------
	/**
	 * Metoden henter flere namespace hvis dette er aktuelt.
	 * 
	 * @param String namespaceURI
	 * @return Iterator
	 */
	public Iterator getPrefixes(String namespaceURI) {
		return getPrefixesIml(namespaceURI, map).iterator();
	}
	//------------------------------------------------------------------------------------
	/**
	 * Metoden henter prefiksene til namespace.
	 * 
	 * @param namespaceURI
	 * @param map
	 * @return
	 */
	protected List getPrefixesIml(String namespaceURI, Map map) {
		List prefixes = new ArrayList();

		Set keys = map.keySet();

		for (Iterator iterator = keys.iterator(); iterator.hasNext();)	{

			String prefix = (String) iterator.next();

			String uri = (String) map.get(prefix);

			if (uri.equals(namespaceURI))
				prefixes.add(prefix);

		}
		return prefixes;
	}

}