/**
 * Klassen pner XML-dokumentet og parser det ved hjelp av DOMParser.
 * Det parsete XML-dokumentet lagres som Document. Inneholder ogs 
 * metode for XPath.
 * 
 * @author Thomas Skjlberg
 */
package filmweb;


import java.io.FileInputStream;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.xerces.jaxp.DocumentBuilderFactoryImpl;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class Parsing {
	
	Console console;
	ComponentList componentList;
	public Parsing(Console c){
		console = c;
	}
	//--------------------------------------------------------------------------------------------
	/**
	 * Metoden pner XML-dokuemntet, parser det ved hjelp av DOMParser
	 * og lukker filen igjen. Det parsede dokumentet lagres som Document.
	 * 
	 * @param String xmlSti
	 * @return Document
	 * @throws Exception
	 */
	public Document load(String xmlSti) throws Exception{
		DocumentBuilderFactoryImpl factory = new DocumentBuilderFactoryImpl();
		factory.setIgnoringElementContentWhitespace(true);
		factory.setNamespaceAware(true);
		DOMParser parser = new DOMParser();
		  
		FileInputStream filein = new FileInputStream(xmlSti); 
		parser.parse(new InputSource(filein));
		return parser.getDocument();
	}
	//-------------------------------------------------------------------------------------------
	/**
	 * Metoden doXPath lar XPath finne interessante noder.
	 * http://java.sun.com/developer/technicalArticles/xml/validationxpath/
	 * 
	 * contexten er snn at det er samsvar mellom namespacet i bruk i dokumentent og 
	 * det namespacet du mener du bruker i XPath-uttrykket
	 * 
	 * @param document
	 * @param path
	 * @return
	 * @throws XPathExpressionException
	 */
	public NodeList doXPath(Node document, String path) throws XPathExpressionException {

		XPath xpath = XPathFactory.newInstance().newXPath();
		
		NamespaceContextImpl context = new NamespaceContextImpl();
		context.setNamespace("did", "urn:mpeg:mpeg21:2002:02-DIDL-NS");
		context.setNamespace("dii", "urn:mpeg:mpeg21:2002:01-DII-NS");
		context.setNamespace("dip", "urn:mpeg:mpeg21:2005:01-DIP-NS");
		context.setNamespace("dia", "urn:mpeg:mpeg21:2003:01-DIA-NS");
		context.setNamespace("mpeg7", "urn:mpeg:mpeg7:schema:2001");
		
		xpath.setNamespaceContext(context);
		
		// gjr sprringen		
		NodeList nodes = (NodeList)xpath.evaluate(path, document, XPathConstants.NODESET); 			
		
		//System.out.println("XPath " + path + " got " + nodes.getLength() + " nodes from node " + document.getNodeName());
		return nodes;
		
	}

}
