/**
 * Denne klassen lager en liste med komponenter som skal presenteres
 * i GUI. Den lager listen slik av hvis bde bilde og tekst skal vises
 * legges dette i samme element slik at du kommer p samme linje. Er 
 * det flere liste-elementer enn strrelse av vindu vil en scroll synes
 * i vinduet.
 * 
 * @author Thomas Skjlberg
 */
package grafikk;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.ImageProducer;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.ListCellRenderer;

public class DescriptorArrayList extends JList implements ListCellRenderer {

	/**
	 * Deklarering av variabler:
	 */
	private static final long serialVersionUID = 1L;
	Vector descriptors = null;
	Vector components = null;
	DefaultListModel model = new DefaultListModel();
	
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden setContents henter innholdet i alle Components og Descriptors.
	 * 
	 * @param Vector descriptors, vektor med Descriptorer
	 * @throws IOException
	 */
	public void setContents(Vector descriptors) throws IOException {
		
		System.out.println("Set content to " + descriptors.size());
		this.descriptors = descriptors;
		this.components = new Vector();
		
		for(int i = 0; i < descriptors.size(); i++) {
			Component component = createListElement((Vector)descriptors.elementAt(i));
			components.addElement(component);
		}
		
		model.setSize(descriptors.size());
		setModel(model);
		setCellRenderer(this);
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Metodene createListElement lager en liste over alle Descriptorene.
	 * 
	 * @param Vector descriptors, vektor med Descriptorer
	 * @return Component
	 * @throws MalformedURLException
	 */
	public Component createListElement(Vector descriptors) throws MalformedURLException {
		JPanel panel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		panel.setLayout(gb);

		Vector components = new Vector();
		for(int i = 0; i < descriptors.size(); i++) {
			Object[] desc = (Object[])descriptors.elementAt(i);
			String mimeType = (String)desc[0];
			
			if(desc[1] != null) { // get contents
				 if(mimeType.startsWith("image/")) {
						desc[2] = getNullImage(new URL((String)desc[1]));
						
				 }
				 else if(mimeType.startsWith("text/")) {
					desc[2] = getText(new URL((String)desc[1]));
				 }
			}
			Component comp = createElement(desc);
			if(comp != null) {
				components.addElement(comp);
			}
		}
		
		for(int i = 0; i < components.size(); i++) {
	
			Component comp = (Component)components.elementAt(i);
			
			GridBagConstraints c;
			int index = descriptors.size() - 1 - i;
			if(index > 1) {
				c = getConstraints(1, !(comp instanceof ImageViewer));
			} else if(index == 1) {
				c = getConstraints(GridBagConstraints.RELATIVE, !(comp instanceof ImageViewer));
			} else {
				c = getConstraints(GridBagConstraints.REMAINDER, !(comp instanceof ImageViewer));
			}
			gb.setConstraints(comp, c);
			panel.add(comp);
		}
		return panel;
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden GetConstraints bestemmer hvordan innholdet skal presenteres i GUI.
	 * Det er nskelig at teksten starter vest i vinduet og at elementene
	 * legge horisontalt.
	 * 
	 * @param int gridWidth
	 * @param boolean fill
	 * @return GridBagConstraints
	 */
	public GridBagConstraints getConstraints(int gridWidth, boolean fill){
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		if(fill) {
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.weightx = 1;
		} else {
			gbc.weightx = 0;
		}
		gbc.gridwidth = gridWidth;
		gbc.weighty = 0;
		
		return gbc;
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden createElement lager bilde og tekst i et element, hvis dette 
	 * er nskelig.
	 * 
	 * @param Object[] descriptor, tabell av Descriptorer
	 * @return Component
	 */
	public Component createElement(Object[] descriptor) {
		String mimeType = (String)descriptor[0];
		if(mimeType.startsWith("image/")) {
			Image image = (Image)descriptor[2];
			return new ImageViewer(image);
		}
		else if(mimeType.startsWith("text/")  && !mimeType.startsWith("text/xml")) {

			JTextPane title = new JTextPane();
		    title.setEditable(false);
		    title.setOpaque(false);
			String text = (String)descriptor[2];
		    title.setContentType(mimeType);
		    title.setText(text);	  

		    return title;
		} 
		return null;
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden getText henter teksten i en eventuell URL.
	 * 
	 * @param URL url, dette er URL
	 * @return String
	 */
	public String getText(URL url) {
		try {
	        // Read all the text returned by the server
	        Reader reader = new InputStreamReader(url.openStream());
	        
	        char[] charBuffer = new char[1000];
	        StringBuffer buffer = new StringBuffer();
	        
	        int read = reader.read(charBuffer);
	        while(read != -1) {
	        	buffer.append(charBuffer, 0, read);
	        	read = reader.read(charBuffer);
	        }
	        return buffer.toString();
	    } catch (MalformedURLException e) {
	    } catch (IOException e) {
	    }
	    return null;
	}
	//-------------------------------------------------------------------------------------
	
	/**
	 * Metoden gr gjennom listen og gjengir den som Component.
	 * 
	 * @param JList list, GUIliste
	 * @param Object value, verdi for display
	 * @param int index, celleindeks
	 * @param boolean isSelected, er cellen valgt
	 * @param cellHasFocus the list and the cell have the focus
	 * @return Component
	 */
	public Component getListCellRendererComponent(JList list, Object value,
			int index,boolean isSelected,boolean cellHasFocus) 
	{
		
		Component comp = (Component)components.elementAt(index);
		if (isSelected) {
			comp.setBackground(list.getSelectionBackground());
			comp.setForeground(list.getSelectionForeground());
			comp.setBackground(Color.CYAN);
		} else {
			comp.setBackground(list.getBackground());
			comp.setForeground(list.getForeground());
			comp.setBackground(Color.BLUE);
		}
		
		return comp;
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden returnerer et tomt bilde.
	 * 
	 * @param url
	 * @return Image
	 */
	public Image getNullImage(URL url) {
		Image image = null;
		try {
			Object obj = url.getContent();
			if(obj instanceof ImageProducer) {
				image = Toolkit.getDefaultToolkit().createImage((ImageProducer)obj);
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return image;
	}
	
}