/**
 * Klassen viser bilde i GUI. Klassen kalles fra klassen Descriptor-
 * ArrayList og gjr det mulig  vise bilde i GUI.
 * 
 * @author Thomas Skjlberg
 */
package grafikk;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;
import java.net.URL;

import javax.swing.JComponent;

/** 
* Component for viewing an Image. Simplified version of Symantec ImageViewer 
* displays an Image. Box grows or shrinks to fit. 
* @author Roedy Green 
*/
public class ImageViewer extends JComponent {
	/**
	 * default Constructor
	 */
	private static final long serialVersionUID = 1L;

	public ImageViewer() {
		image = null;
	}

	/** 
	* Constructor with Image. 
	* 
	* @param image the Image to be displayed . 
	* See the Java glossary under Image for ways to create an Image from a file. 
	*/
	public ImageViewer(Image image) {
		super();
		setImage(image);
		//setOpaque(false);
	}

	public static Image sumImages(Image[] images, Component tracker) {
		if(images.length == 0) return null;
		Image sum = images[0];
		for(int teller = 1; teller < images.length; teller++) {
			sum = getButtonImage(sum, images[teller], tracker);
		}
		return sum;
	}

	public static Image getButtonImage(Image image, Image button, Component tracker) {
		if(image == null | button == null) return null;
			MediaTracker mt = new MediaTracker(tracker);
			//add the image to the media tracker's list
			mt.addImage(image, 1);
			mt.addImage(button, 2);
			//wait for all images in list to load before proceeding
			try {
				mt.waitForAll();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
	
			//serialize to bytes
	
			int imageWidth = image.getWidth(tracker);
			int imageHeight = image.getHeight(tracker);
			
			if(imageWidth < 0 | imageHeight < 0) return button;
			
			int[] image_pixels = new int[imageWidth * imageHeight];
		
			int buttonWidth = button.getWidth(tracker);
			int buttonHeight = button.getHeight(tracker);
			int[] button_pixels = new int[buttonWidth*buttonHeight];		
			
			PixelGrabber pg1 = new PixelGrabber(image, 0, 0, imageWidth, imageHeight, image_pixels, 0, imageWidth);
			PixelGrabber pg2 = new PixelGrabber(button, 0, 0, buttonWidth, buttonHeight, button_pixels, 0, buttonWidth);
			//copy pixels into array
			try {
				pg1.grabPixels();
				pg2.grabPixels();			
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
	
			// kopier inn informasjonen
			
			int newWidth = buttonWidth+imageWidth;
			int newHeight = imageHeight;
			if(newHeight < buttonHeight) newHeight = buttonHeight;
	
			int[] new_pixels = new int[newWidth*newHeight];
			
			// kopier inn bildet
			for(int hoyde = 0; hoyde < imageHeight; hoyde++) {
				for(int bredde = 0; bredde < imageWidth; bredde++) {
					new_pixels[hoyde*newWidth+bredde] = image_pixels[hoyde*imageWidth+bredde];
				}
			}
			
			int offset = (int)(newHeight-buttonHeight)/2;
			//System.out.println(offset + " " + buttonWidth + " " + imageWidth);
			
			
			for(int hoyde = 0; hoyde < buttonHeight; hoyde++) {
				for(int bredde = 0; bredde < buttonWidth; bredde++) {
					new_pixels[imageWidth+(offset+hoyde)*newWidth+bredde] = button_pixels[hoyde*buttonWidth+bredde];
				}
			}
			  //rebuild into image
	
			  //use MemoryImageSource class to create image
			Image temp_image = tracker.createImage(new MemoryImageSource(newWidth, newHeight, new_pixels, 0, newWidth));
	
			  //use MediaTracker again to wait for image to load
			mt.addImage(temp_image, 1);
			try {
				mt.waitForAll();
			} catch (InterruptedException e2) {
				e2.printStackTrace();
			}
			return temp_image;
	}


	/** 
	* Set or change the current Image to display. 
	* setImage does a MediaTracker to ensure the Image is loaded. 
	* You don't have to. 
	* 
	* @param image the new Image to be displayed. 
	* 
	*/
	public void setImage(Image image) {
		this.image = image;
		if (image != null) {
			MediaTracker tracker;
			try { // wait until image is fully loaded. 
				tracker = new MediaTracker(this);
				tracker.addImage(image, 0);
				tracker.waitForID(0);
			} catch (InterruptedException e) {
			}
		} else {
			repaint();
		}
	}

	/** 
	* Get the Image currently being displayed. 
	* 
	* @return the Image currently displayed or null if no Image 
	*/
	public Image getImage() {
		return image;
	}

	/** 
	* Paints this component using the given graphics context. 
	* 
	* @param g Graphics context where to paint, e.g. to screen, printer, RAM. 
	*/
	public void paint(Graphics g) {
		super.paint(g);
		// get size of box we have to draw in 
		Dimension dim = getSize();
		if (image != null) {
			/* center Image in box, normally should exactly fill the box. 
			If we overflow, no problem, drawImage will clip. */
			int imageWidth = image.getWidth(this);
			int imageHeight = image.getHeight(this);

			g.drawImage(image, /* Image to draw */
			 0, //(dim.width - imageWidth) / 2, /* x */
			 0, //(dim.height - imageHeight) / 2, /* y */
			imageWidth, /* width */
			imageHeight, /* height */
			this); /* this ImageViewer component */

		} else {
			/* we have no Image, clear the box */
			g.clearRect(0, 0, dim.width, dim.height);
		}
	}

	/** 
	* Preferred Layout size. 
	* 
	* @return the recommended dimensions to display the Image. 
	*/
	public Dimension getPreferredSize() {
		if (image != null) {
			/* should just fit the Image */
			return (new Dimension(image.getWidth(this), image.getHeight(this)));
		} else { /* empty square as a placeholder */
			return new Dimension(0, 0);
		}
	}

	/** 
	* Minimum layout size. 
	* 
	* @return the minimum dimensions to properly display the Image 
	*/
	public Dimension getMinimumSize() {
		return getPreferredSize();
	}

	/** 
	* Image that this viewer is currently displaying. 
	*/
	private Image image;

	public Image getNullImage(URL target) {
		Image retur = null;
		try {
			//retur = getImage(target);
			retur =  Toolkit.getDefaultToolkit().getDefaultToolkit().createImage(target);
		} catch(Exception e) {
			e.printStackTrace();
		}
		return retur;
	}
	
} // end class ImageViewer 
