/**
 * Klassen inneholder grafikk og presenterer GUI for filmliste. Ved hjelp
 * av klassen DescriptorArrayList genereres listen. Klassen kalles fra 
 * klassen DIP i pakken mpeg21.
 * 
 * @author Thomas Skjlberg
 */
package grafikk;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import filmweb.ToButtonInterface;

public class MainJFrame2 extends JFrame implements ActionListener, WindowListener, Runnable {

	/**
	 * Deklarerer variabler:
	 */
	private static final long serialVersionUID = 1L;
	ToButtonInterface toButtonInterface = null;
	
    static JPanel panel = null;
    static CardLayout cardLayout = new CardLayout();
    JLabel statusBar = null;     
    
    JButton leftButton = new JButton();
    JButton rightButton = new JButton();
    
	JEditorPane title = new JEditorPane();
    
	Thread actionEventThread = null;
	ActionEvent actionEvent = null;
	
	DescriptorArrayList list = null;
	//----------------------------------------------------------------------------------
	/**
	 * Konstruktr av klassen.
	 * @param toButtonInterface
	 */
	public MainJFrame2(ToButtonInterface toButtonInterface) {
		this.toButtonInterface = toButtonInterface;
		
		addWindowListener(this);
		
		GridBagLayout gb = new GridBagLayout();
		getContentPane().setLayout(gb);
		getContentPane().setBackground(Color.BLUE);
		
		// fix full line layout constraints
		GridBagConstraints fullLine = getFullLineConstraints();
		//GridBagConstraints halfLine = gethalfLineConstraints();
		
		// the title
		title.setEditable(false);
		title.setBackground(Color.BLUE);
		gb.setConstraints(title, fullLine);
		getContentPane().add(title);
		
		// create the card frame
		panel = new JPanel();
		panel.setLayout(cardLayout);
		panel.setBackground(Color.BLUE);
		gb.setConstraints(panel ,getFullPaneConstraints());
		getContentPane().add(panel);

		// buttons
		Container buttonHolder = new Container();
		GridLayout buttonHolderLayout = new GridLayout(1, 1);
		buttonHolder.setLayout(buttonHolderLayout);
		
		leftButton.setText("Avbryt");
		leftButton.addActionListener(this);
		rightButton.setText("Velg");
		rightButton.addActionListener(this);
		
		buttonHolder.add(leftButton);
		buttonHolder.add(rightButton);

		gb.setConstraints(buttonHolder, fullLine);
		getContentPane().add(buttonHolder);
		
		setSize(240, 268);
		setVisible(true);
	}
	//-----------------------------------------------------------------------------------
	public void addContent(Component component, String alias) {
		panel.add(component, alias);
	}
	//-----------------------------------------------------------------------------------
	public void showContent(String alias) {
		cardLayout.show(panel, alias);
		panel.validate();
	}
	//-----------------------------------------------------------------------------------
	public void setHeader(String html, String mimeType) {
		title.setContentType(mimeType);
		title.setText(html);
	}
	//-----------------------------------------------------------------------------------	
	public static GridBagConstraints getFullLineConstraints() {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.weightx = 1;
		gbc.weighty = 0;
		return gbc;
	}
	//-----------------------------------------------------------------------------------
	public static GridBagConstraints getFullPaneConstraints() {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.weightx = 1;
		gbc.weighty = 1;
		return gbc;
	}
	//-----------------------------------------------------------------------------------
	public void setStatus(String status) {
		if(statusBar == null) return;
		statusBar.setText(status);
		System.out.println(status);
		statusBar.repaint();
	}
	//-----------------------------------------------------------------------------------
	public void actionPerformed(ActionEvent e) {
		if(actionEventThread != null && actionEventThread.isAlive()) {
			return;
		}
		actionEvent = e;
		actionEventThread = new Thread(this);
		actionEventThread.start();
		
	}
	//-----------------------------------------------------------------------------------
	/**
	 * Metoden pner en liste over nsket element som skal presenteres
	 * i menyvinduet.
	 * 
	 * @param Vector v, liste med elementer som skal vises i GUI
	 * @param String text, tekst som skal vises i GUI
	 * @param String mimeType, type element
	 * @throws IOException
	 */
	public void openList(Vector v, String text, String mimeType) throws IOException {
		JPanel panel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		panel.setLayout(gb);
		panel.setBackground(Color.BLUE);
		
		JEditorPane label= new JEditorPane();
		label.setContentType(mimeType);
		label.setEditable(false);
		label.setText(text);
		label.setBackground(Color.BLUE);
		
		gb.setConstraints(label, getFullLineConstraints());
		panel.add(label);
		
		if(list == null) {
			list = new DescriptorArrayList();
			
			JScrollPane scrollPane = new JScrollPane(list);
			
			gb.setConstraints(scrollPane, getFullPaneConstraints());
			panel.add(scrollPane);
			
			addContent(panel, "list");
		}
		list.setContents(v);
		showContent("list");
	}
	//-----------------------------------------------------------------------------------
	public int getSelectedIndex() {
		return list.getSelectedIndex();
	}

	public void windowOpened(WindowEvent e) {
	}

	public void windowClosing(WindowEvent e) {
		System.out.println("Window closing!");		
		System.exit(0);
	}

	public void windowClosed(WindowEvent e) {
	}

	public void windowIconified(WindowEvent e) {
	}

	public void windowDeiconified(WindowEvent e) {
	}

	public void windowActivated(WindowEvent e) {
	}

	public void windowDeactivated(WindowEvent e) {
	}
	//-------------------------------------------------------------------------------------------
	/**
	 * Metoden run kjrer en trd som sjekker om knappene i menyen er trykt.
	 */
	public void run() {
		if(leftButton == actionEvent.getSource()) {
			toButtonInterface.leftButton();
		} else if(rightButton == actionEvent.getSource()) {
			if(list.getSelectedIndex() != -1){
			setStatus("Component chosen!");
			}
			else{
				setStatus("Choose a component!");
			}
			toButtonInterface.rightButton();
			
		}
		actionEventThread = null;
	}

}

