/**
 * Klassen spiller film i QuickTime for Java. Klassen kalles fra metoden 
 * play i klassen DIP. Spilleren spiller bde synkront og asynkront, dette
 * bestemmes i play. Spilleren returnerer ogs status knyttet til PlayStatus.
 * 
 * @author Trude
 */
package grafikk;

import java.awt.*;
import java.awt.event.*;

import javax.swing.JFrame;

import mpeg21.PlayStatus;
import mpeg21.PlayStatusImpl;



import quicktime.*;
import quicktime.std.*;
import quicktime.app.view.*;
import quicktime.std.movies.*;
import quicktime.std.movies.media.*;



public class PlayMovie extends JFrame implements Errors, WindowListener, PlayStatus {

	/**
	 * Deklarering av variabler:
	 */
	private static final long serialVersionUID = 1L;
	private Movie m;
	private MovieController mc;	
	private QTComponent myQTCanvas;
	
	protected Object lock = null;
    protected boolean isLocked = false;
	public boolean asyncronize;
	
	boolean released = false;
	
	PlayStatusImpl playStatus = null;
	//--------------------------------------------------------------------------------
	/**
	 * Konstruktr av klassen.
	 * 
	 * @param title
	 * @param url
	 * @param async
	 * @throws Exception
	 */
	public PlayMovie (String title, String url, boolean async) throws Exception {
		super(title);
		
		asyncronize = async;
		
		if(lock == null) lock = new Object();
		
		addWindowListener(this);
		getContentPane().setLayout (new BorderLayout());		
		init();
		createNewMovieFromURL(url);
		
	}
	//--------------------------------------------------------------------------------
    /**
     * Metoden initierer spilleren.
     * 
     * @throws RuntimeException
     */
	public void init () throws RuntimeException {
		
		try {
			//synchronized(QTStreamingApplet.lock) {
				if (QTSession.isInitialized() == false)
					QTSession.open();
			//}		
		} catch (QTException qtE) {
				//in this case the only QTException is in QTSession.open()
			throw new RuntimeException (qtE.getMessage());		
		}
	}	
	//-------------------------------------------------------------------------------
	/**
	 * Metoden start starter spillern og setter status TIMEPLAY.
	 * 
	 * @throws InterruptedException
	 */
	public PlayStatus start () throws InterruptedException { 
		//System.out.println("Starter avspilling!");
		 
		try {
			if (m != null) {
				m.setRate(1f);
				playStatus = new PlayStatusImpl(m);				
				playStatus.setStatus(PlayStatus.TIMEPLAY);
			}				
		} catch (QTException err) {
			err.printStackTrace();

			playStatus.setStatus(PlayStatus.RELEASED);
		}
		
		if(asyncronize == true){
			lock();
		}
	
		
		return playStatus;
	}
	//-------------------------------------------------------------------------------
	/**
	 * Metoden pause fanger opp pause i avspillingen og setter status STATICPLAY.
	 *
	 */
	public void pause(){
		System.out.println("Her er det pause");
		try{
			if (m != null){
				m.setRate(0);
				playStatus.setStatus(PlayStatus.STATICPLAY);
			}
		}catch(QTException err){
			err.printStackTrace();
		}
	}
	//--------------------------------------------------------------------------------
	/**
	 * Metoden stop stopper avspillingen og setter status RELEASED.
	 */
	public void stop () { 
			try{
		if (myQTCanvas != null) {
			myQTCanvas.setMovie(null);
			released = true;
			playStatus.setStatus(PlayStatus.RELEASED);
		}
		} catch (QTException qtE) {
				//in this case the only QTException is in QTSession.open()
			throw new RuntimeException (qtE.getMessage());		
		}
	}
	//--------------------------------------------------------------------------------
	/**
	 * Metoden destroy lukker spilleren nr x trykkes.
	 */
	public void destroy () {
			QTSession.close();
	}
	/**
	 * Metoden destroy2 lukker spilleren slik at nr browseren
	 * restarter appleten, s er gammel informasjon slettet.
	 *
	 */ 	
	public void destroy2() {
	    while(QTSession.isInitialized()) {
	        QTSession.close();
	    }
	}
	//------------------------------------------------------------------------------	
	/**
	 * Lager presentable objekter fra URL
	 * 
	 * @param selURL
	 * @throws QTException
	 */
	private void createNewMovieFromURL(String selURL) throws QTException {
	    
		DataRef dRef = new DataRef(selURL);
		m = Movie.fromDataRef (dRef,StdQTConstants.newMovieActive);
		mc = new MovieController(m);
		mc.forceTimeTableUpdate();
        
		if (myQTCanvas == null) {
			myQTCanvas = QTFactory.makeQTComponent(mc);
			getContentPane().add((Component)myQTCanvas, "Center");	
		}else {
			myQTCanvas.setMovieController (mc);
		}
		pack();
		
	}
	//---------------------------------------------------------------------------
	/**
	 * Metoden unlock srger for synkronisering for avspilling av film.
	 */
	public void unlock() {
        synchronized (lock) {
        	System.out.println("unlock");
            lock.notifyAll();
        }
    }
	//----------------------------------------------------------------------------
	/**
	 * Metoden lock lser opp synkronkjringen. 
	 * @throws InterruptedException
	 */
	public void lock() throws InterruptedException {
	    synchronized (lock) {
	    	System.out.println("Lock-funksjon");
	    	isLocked = true;
            lock.wait();
            isLocked = false;
	    }
	}
	
	public java.awt.im.InputContext getInputContext() {
		return null;
	}
	
	public void	windowActivated(WindowEvent e) { 
	}
				
	public void	windowClosed(WindowEvent e) {
	}
				
	public void	windowClosing(WindowEvent e) {
		this.setVisible(false);
		stop();
		unlock();
	}			
	
	public void	windowDeactivated(WindowEvent e){
	} 
				
	public void	windowDeiconified(WindowEvent e){ 
	}
				
	public void	windowIconified(WindowEvent e) {
	}
	
	public void	windowOpened(WindowEvent e){ 
	}
	//--------------------------------------------------------------------------
	/**
	 * Metoden getStatus henter ut status.
	 */
	public int getStatus() {
		if(released){
			return RELEASED;
		}
		try{
			if(m.getRate() > 0){
				return TIMEPLAY;
			}
			return STATICPLAY;
		}
		catch(QTException err){
			err.getMessage();
		}
		
	return 0;
	}
	
}
