/**
 * Klasse for Filmweb Nyhetsbrev. Presenterer GUI for innskriving av
 * epostadresse. Nr epostadresse er skrevet inn kommer den frem i vinduet
 * s man kan kontollere om det er riktig adresse. Klassen er hentet
 * fra Swing og redigert av Trude.
 * 
 * @author Trude
 */
package grafikk;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

import filmweb.Console;

public class TextInput extends JPanel implements ActionListener, FocusListener {
	
    /**
	 * Deklarering av variabler:
	 */
	private static final long serialVersionUID = 1L;
	JTextField emailField;
    JTextField title;

    boolean addressSet = false;
    Font regularFont, italicFont;
    JLabel addressDisplay;
    final static int GAP = 10;

    //------------------------------------------------------------------------------------
    /**
     * Konstruktr av klassen TextInput
     * @param console 
     */
    public TextInput(Console console) {
        setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

        JPanel leftHalf = new JPanel() {
            /**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			//Don't allow us to stretch vertically.
            public Dimension getMaximumSize() {
                Dimension pref = getPreferredSize();
                return new Dimension(Integer.MAX_VALUE,
                                     pref.height);
            }
        };
        leftHalf.setLayout(new BoxLayout(leftHalf,
                                         BoxLayout.PAGE_AXIS));
        leftHalf.add(createEntryFields());
        leftHalf.add(createButtons());
        

        add(leftHalf);
        add(createAddressDisplay());
    }
    
    //------------------------------------------------------------------------------
    /**
     * Metoden createButtons lager kanppene p panelet.
     * 
     * @return JComponent
     */
    protected JComponent createButtons() {
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.TRAILING));
        panel.setBackground(Color.BLUE);
        JButton button = new JButton("Set email address");
        button.addActionListener(this);
        panel.add(button);

        button = new JButton("Clear email address");
        button.addActionListener(this);
        button.setActionCommand("clear");
        panel.add(button);

        //Match the SpringLayout's gap, subtracting 5 to make
        //up for the default gap FlowLayout provides.
        panel.setBorder(BorderFactory.createEmptyBorder(0, 0,
                                                GAP-5, GAP-5));
        return panel;
    }
    //---------------------------------------------------------------------------------
    /**
     * Metoden actionPerformes kalles nr brukeren trykker
     * p kanppen eller trykker enter.
     * 
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        if ("clear".equals(e.getActionCommand())) {
            addressSet = false;
            emailField.setText("");
            
        } else {
            addressSet = true;
        }
        updateDisplays();
    }
    
    //----------------------------------------------------------------------------
    /**
     * Metoden updateDisplays oppdaterer dsiplayet.
     *
     */
    protected void updateDisplays() {
        addressDisplay.setText(formatAddress());
        if (addressSet) {
            addressDisplay.setFont(regularFont);
        } else {
            addressDisplay.setFont(italicFont);
        }
    }
    //----------------------------------------------------------------------------
    /**
     * Metoden createAdressDisplay lager adressefeltet for epost.
     * 
     * @return JComponent
     */
    protected JComponent createAddressDisplay() {
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBackground(Color.BLUE);
        addressDisplay = new JLabel();
        addressDisplay.setHorizontalAlignment(JLabel.CENTER);
        regularFont = addressDisplay.getFont().deriveFont(Font.PLAIN,
                                                            16.0f);
        italicFont = regularFont.deriveFont(Font.ITALIC);
        updateDisplays();

        //Lay out the panel.
        panel.setBorder(BorderFactory.createEmptyBorder(
                                GAP/2, //top
                                0,     //left
                                GAP/2, //bottom
                                0));   //right
        panel.add(new JSeparator(JSeparator.VERTICAL),
                  BorderLayout.LINE_START);
        panel.add(addressDisplay,
                  BorderLayout.CENTER);
        panel.setPreferredSize(new Dimension(200, 150));

        return panel;
    }
    
    //------------------------------------------------------------------------------
    /**
     * Metoden formatAddress formaterer epostadressen.
     * 
     * @return String
     */
    protected String formatAddress() {
        if (!addressSet) return "No email set.";

        String street = emailField.getText();
        String empty = "";

        if ((street == null) || empty.equals(street)) {
            street = "<em>(no email specified)</em>";
        }


        StringBuffer sb = new StringBuffer();
        sb.append("<html><p align=center>");
        sb.append(" ");
        sb.append(street);
        sb.append("<br>");
        sb.append("</p></html>");

        return sb.toString();
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Metoden createFormatter lager en MaskFormatter.
     * 
     * @param s
     * @return MaskFormatter
     */
    protected MaskFormatter createFormatter(String s) {
        MaskFormatter formatter = null;
        try {
            formatter = new MaskFormatter(s);
        } catch (java.text.ParseException exc) {
            System.err.println("formatter is bad: " + exc.getMessage());
            System.exit(-1);
        }
        return formatter;
    }

    //------------------------------------------------------------------------------
    /**
     * Metoden focusGained kalles nr en av feltene fr fokus
     * slik at de kan velge det fokuserte feltet.
     */
    public void focusGained(FocusEvent e) {
        Component c = e.getComponent();
        if (c instanceof JFormattedTextField) {
            selectItLater(c);
        } else if (c instanceof JTextField) {
            ((JTextField)c).selectAll();
        }
    }
    
    //------------------------------------------------------------------------------
    /**
     * Workaround for formatted text field focus side effects.
     * @param c
     */
    protected void selectItLater(Component c) {
        if (c instanceof JFormattedTextField) {
            final JFormattedTextField ftf = (JFormattedTextField)c;
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    ftf.selectAll();
                }
            });
        }
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Needed for FocusListener interface.
     * 
     * @param e
     */
    public void focusLost(FocusEvent e) { } 

    //------------------------------------------------------------------------------------
    /**
     * Metoden createEntryFields lager listen med de nskede
     * elementer i GUI.
     * 
     * @return
     */
    protected JComponent createEntryFields() {
        JPanel panel = new JPanel(new SpringLayout());
        panel.setBackground(Color.BLUE);
        
        String[] labelStrings = {
        		
        		"Email address: "
        };

        JLabel[] labels = new JLabel[labelStrings.length];
        JComponent[] fields = new JComponent[labelStrings.length];
        int fieldNum = 0;

        //Create the text field and set it up.
        emailField  = new JTextField();
        emailField.setColumns(20);
        fields[fieldNum++] = emailField;
        
        //Associate label/field pairs, add everything,
        //and lay it out.
        for (int i = 0; i < labelStrings.length; i++) {
            labels[i] = new JLabel(labelStrings[i],
                                   JLabel.TRAILING);
            labels[i].setLabelFor(fields[i]);
            panel.add(labels[i]);
            panel.add(fields[i]);

            //Add listeners to each field.
            JTextField tf = null;
            if (fields[i] instanceof JSpinner) {
                tf = getTextField((JSpinner)fields[i]);
            } else {
                tf = (JTextField)fields[i];
            }
            tf.addActionListener(this);
            tf.addFocusListener(this);
        }
        SpringUtilities.makeCompactGrid(panel,
                labelStrings.length, 2,
                GAP, GAP, //init x,y
                GAP, GAP/2);//xpad, ypad
        return panel;
    }

    //-----------------------------------------------------------------------------
    /**
     * 
     * @param spinner
     * @return
     */
    public JFormattedTextField getTextField(JSpinner spinner) {
        JComponent editor = spinner.getEditor();
        if (editor instanceof JSpinner.DefaultEditor) {
            return ((JSpinner.DefaultEditor)editor).getTextField();
        } else {
            System.err.println("Unexpected editor type: "
                               + spinner.getEditor().getClass()
                               + " isn't a descendant of DefaultEditor");
            return null;
        }
    }

    /**
     * Lager GUI og viser den.
     */
    public void createAndShowGUI() {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        
        JFrame frame = new JFrame("3G Filmweb");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Create and set up the content pane.
        JComponent newContentPane = new TextInput(null);
        newContentPane.setOpaque(true); //content panes must be opaque
        newContentPane.setBackground(Color.BLUE);
        
        frame.setContentPane(newContentPane);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }

}

