/**
 * Klassen presenterer GUI med tekst. Vinduet inneholder kun teksten som
 * blir presentert. Er teksten for stor for vinduet vises scroll. Klassen
 * kalles fra play i DIP i pakken mpeg21.
 * 
 * @author Trude
 */
package grafikk;

import java.awt.*;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;

import javax.swing.*;

public class TextView extends JFrame {

	/**
	 * Deklarering av variabler:
	 */	
	private static final long serialVersionUID = 1L;
    JTextPane textPane;
    static final int MAX_CHARACTERS = 300;
    JTextArea changeLog;
    String newline = "\n";
    HashMap actions;
    
    //-------------------------------------------------------------------------------------
    /**
     * Konstruktr av klassen Textview
     * 
     * @param path
     */
    public TextView(String path) {
        super("3G Filmweb");
        
        //Lay out the text controls and the labels.
        JPanel textControlsPane = new JPanel();
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();

        textControlsPane.setLayout(gridbag);
        
        c.gridwidth = GridBagConstraints.REMAINDER; //last
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1.0;
        
        JTextPane textPane = createTextPane(path);
        textPane.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder(""),
                BorderFactory.createEmptyBorder(5,5,5,5)));
        textPane.setPreferredSize(new Dimension(300, 200));
        textPane.setMinimumSize(new Dimension(10, 10));
        textPane.setBackground(Color.BLUE);
        
        JScrollPane paneScrollPane = new JScrollPane(textPane);
        paneScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        add(paneScrollPane, BorderLayout.LINE_END);
      
    }
    
    //--------------------------------------------------------------------------------------
    /**
     * Metoden createTextPane henter inn referansen til tekst-dokumentet
     * og sjekker at den er gyldig.
     *  
     * @param path
     * @return JTextPane
     */
    public JTextPane createTextPane(String path) {
        JTextPane textPane = new JTextPane();
        textPane.setEditable(false);

        try {
			URL textURL = new URL(path);
			if (textURL != null) {
	            try {
	                textPane.setPage(textURL);
	            } catch (IOException e) {
	                System.err.println("Attempted to read a bad URL: " + textURL);
	            }
	        }
			else {
	            System.err.println("Couldn't find file: TextURL");
	        }
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		}

        return textPane;
    }
    
    //--------------------------------------------------------------------------------------
    /**
     * Lager GUI og viser den. 
     * 
     * @param path
     */
    public static void createAndShowGUI(String path) {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        TextView frame = new TextView(path);
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }

}