/**
 * Klassen DIP inneholder DIP-relaterte DIBOer, disse er getObjectMap(),
 * play() og getObject(). De andre metodene er metoder for  sttte 
 * funksjonaliteten i tjenesten.
 * 
 * @author Trude
 */
package mpeg21;


import java.net.MalformedURLException;
import java.net.URL;

import java.util.Vector;

import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import filmweb.ComponentList;
import filmweb.Parsing;
import filmweb.ToButtonInterface;
import grafikk.ImageViewer;
import grafikk.MainJFrame2;
import grafikk.PlayMovie;
import grafikk.TextView;

import quicktime.QTException;
import quicktime.QTSession;


public class DIP implements ToButtonInterface {
	/**
	 * Deklarering av variabler:
	 */
	String ref;
	String mimeType;
	
	MainJFrame2 frame = null;
	ComponentList componentList;
	Parsing parsing;
	
	protected Object lock = new Object();
	protected boolean isLocked = false;
	
	Document document;
	//--------------------------------------------------------------------------------------------------
	/**
	 * Konstruktr av klassen DIP.
	 * 
	 * @param document
	 */
	public DIP(Document document) {
		this.document = document;
		componentList = new ComponentList(null);
		parsing = new Parsing(null);
	}
	//--------------------------------------------------------------------------------------------------
	public MainJFrame2 getFrame() {
		return frame;
	}
	//--------------------------------------------------------------------------------------------------
	/**
	 * Metoden getObjectMap lager en liste med Components knyttet til 
	 * Object:Type ved hjelp av konstruktren til ObjectMap.
	 * 
	 * @param Document didDocument, parset XML-dokument ligger lagret her
	 * @return ObjectMap, ObjectMap bestr av en vektor med ObjectType
	 * @throws XPathExpressionException
	 */
	public ObjectMap getObjectMap(Document didDocument) throws XPathExpressionException{
		
		ObjectMap objectMap = new ObjectMap(didDocument);
		return objectMap;
	}
	//--------------------------------------------------------------------------------------------------
	/**
	 * Metoden play spiller av nsket valg p menylisten. Hvis mimeType er av 
	 * type audio eller video pnes QuickTime-spiller for Java. Ved mimeType 
	 * av type image pnes vindu for visning av bilde. MimeType text pner
	 * vindu for  vise tekst.
	 * 
	 * @param Element element, Elementobjektet som reflekterer DIDL Item-, Component-, eller Descriptor-elementet
	 * @param boolean async, hvis sann, spill elementet asynkronisert, hvis ikke synkronisert
	 * @return PlayStatus-objektet for  identifisere spille-elementet
	 * @throws Exception 
	 */
	public PlayStatus play(Element element, boolean async) throws Exception{
		
		PlayStatus playStatus = null;
		
		NodeList resourceElementList = parsing.doXPath(element, "did:Resource");
		
		for(int i = 0; i < resourceElementList.getLength(); i++) {
			
			Element type = (Element)resourceElementList.item(i);
			mimeType = type.getAttribute("mimeType");
			ref = type.getAttribute("ref");
			
			if(mimeType != null && ref != null) {
				System.out.println("Attempting to open " + mimeType + " " + ref);
				if(mimeType.startsWith("audio/") || mimeType.startsWith("video/")) {
					try { 
						QTSession.open();
						PlayMovie pm = new PlayMovie("3G Filmweb", ref, async);
						pm.show();
						playStatus = pm.start();
					} catch (QTException e) {
						e.printStackTrace();
						QTSession.close ();
						PlayStatusImpl impl = new PlayStatusImpl(null);
						impl.setStatus(PlayStatus.RELEASED);
						playStatus = impl;
					}
				} 
				else if(mimeType.startsWith("image/")){
					System.out.println("Attempting to open " + mimeType + " " + ref);
					
					try {
				        // Create a URL for the image's location
				        URL url = new URL(ref);
				    
				        // Get the image
				        java.awt.Image image = java.awt.Toolkit.getDefaultToolkit().getDefaultToolkit().createImage(url);
				        
				        new ImageViewer(image);
				    } catch (MalformedURLException e) {
				    }
					
				}
				else if(mimeType.startsWith("text/")){
					javax.swing.SwingUtilities.invokeLater(new Runnable() {
			            public void run() {
			            	
			                TextView.createAndShowGUI(ref);
			            }
			        });
					
				}
				else {
					System.out.println("Show " + mimeType);
				}
			} else {
				System.out.println("Could not open element " + i + " " + mimeType + " " + ref);
			}
		}
		
		return playStatus;
	}
//	--------------------------------------------------------------------------------------------------
	/**
	 * Skaffer User en eller flere valg av objekter av typen ObjectTypes. 
	 * Objektene for hver valg er de elementene identifisert i ObjectMap
	 * i Digital Item av gitt ObjectType.
	 * 
	 * @param String[] objectTypes, tabell av ObjectTypes
	 * @param requestMessages, tabell med tekstbeskeder
	 * @return ELement[]
	 */
	public Element[] getObjects(String[] objectTypes, String[] requestMessages){
		
		Vector parentList = new Vector();
		Vector guiList = new Vector();
		String iconPath = null;
		
		try {
			
			ObjectMap om = new ObjectMap(document);
			iconPath = componentList.getFilmwebIcon(document);
			
			for(int i = 0; i < objectTypes.length; i++) {
				
				String objectType = objectType = objectTypes[i];
				//System.out.println(objectType);
				Element[] el = om.getObjects(objectType);
				for(int j = 0; j < el.length; j++) {
					//System.out.println(" " + j + " " + el[j]);
					parentList.addElement(el[j]);
				}
				guiList = componentList.getListOfElement(parentList);
	
				frame = new MainJFrame2(this);
				frame.setHeader("<html><img src =" + iconPath+ "></img></html>", "text/html");
				frame.openList(guiList, requestMessages[i], "text/plain");
			}
			lock();
		} catch (Exception e) {
			e.printStackTrace();
		}

		int selectedIndex = frame.getSelectedIndex();
		
		return new Element[]{(Element)parentList.elementAt(selectedIndex)};
	}
	
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden leftButton utfrer handlinger nr venstre knapp er trykket.
	 */
	public void leftButton() {
		System.exit(0);
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden rightButton utfrer handlinger nr hyre knapp er trykket.
	 */
	public void rightButton() {
		unlock();

	}
	//---------------------------------------------------------------------------
	/**
	 * Metoden unlock lser opp synkronkjringen. 
	 */
	public void unlock() {
        synchronized (lock) {
        	System.out.println("unlock");
            lock.notifyAll();
        }
    }
	//----------------------------------------------------------------------------
	/**
	 * Metoden lock srger for synkronisering for avspilling av film.
	 * 
	 * @throws InterruptedException
	 */
	public void lock() throws InterruptedException {
	    synchronized (lock) {
	    	System.out.println("Lock-funksjon");
	    	isLocked = true;
            lock.wait();
            isLocked = false;
	    }
	}
	//--------------------------------------------------------------------------------------------------
	/**
	 * Metoden alert skriver ut tekst p menyen. Denne er ikke inkludert i 
	 * tjenesten, men er nyttig  ha for testing.
	 * 
	 * @param text
	 */
	public void alert(String text){
		
		System.out.println(text);
	}
	
}
