/**
 * Klassen inneholder funksjoner for ObjectMap. Ikke alle funksjonene fra
 * MPEG-21 del 10 er implementert, men de som var viktige for diplom-
 * oppgaven. Endelig tjeneste benytter ikke alle metodene, men alle er 
 * testet slik at man vet at de fungerer. De var ogs nyttige metoder
 * for  teste underveis i implementeringen.
 * 
 * @author Trude
 */
package mpeg21;

import java.util.Hashtable;
import java.util.Vector;

import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import filmweb.Parsing;


public class ObjectMap {
	
	/**
	 * Deklarering av variabler:
	 */
	Hashtable objectTypes = new Hashtable();
	NodeList objectTypeElementList;
	
	Parsing parsing;
	//-------------------------------------------------------------------------------------------
	/**
	 * Konstruktr av klassen.
	 * 
	 * @param didDocument
	 * @throws XPathExpressionException
	 */
	public ObjectMap(Document didDocument) throws XPathExpressionException{
		
		parsing = new Parsing(null);
		
		objectTypeElementList = parsing.doXPath(didDocument, "//dip:ObjectType");
		
		for(int i = 0; i < objectTypeElementList.getLength(); i++) {
			//System.out.println(i + " " + objectTypeElementList.item(i));
			Element type = (Element)objectTypeElementList.item(i);
			//System.out.println(i + " " + type.getTextContent());
			
			addObjectTypesUnParsed((Element) type.getParentNode().getParentNode().getParentNode(),
			type.getTextContent());
		}
	}
	//--------------------------------------------------------------------------------------------
	/**
	 * Metoden returnerer veriden p antall typer som finnes i
	 * listen objectTypeElementList.
	 * 
	 * @return Integer
	 */
	public Integer getNumberOfTypes(){
		
		Integer retur = new Integer(objectTypeElementList.getLength());
		return retur;
	}
	//------------------------------------------------------------------------------------
	/**
	 * Metoden getTypeName henter navnet p ObjectType til nsket indeks.
	 * 
	 * @param index
	 * @return String
	 */
	public String getTypeName(int index){
		
		Element type = (Element)objectTypeElementList.item(index);
		
		String retur = type.getTextContent();
		return retur;
	}
	//-------------------------------------------------------------------------------------
	/**
	 * Metoden getNumberOfObjects finner antall typer av en
	 * bestemt ObjectType.
	 * 
	 * @param typeName
	 * @return Integer
	 */
	public Integer getNumberOfObjects(String typeName){
		
		if(objectTypes.containsKey(typeName)){
			Vector objectTypeTable = (Vector) objectTypes.get(typeName);
			Integer retur = new Integer(objectTypeTable.size());
			return retur;
			
		}
		
		return null;
	}
	//-------------------------------------------------------------------------------
	/**
	 * Metoden getObject henter objectet for bestemt type og indeks.
	 * 
	 * @param typeName
	 * @param index
	 * @return
	 */
	public Element getObject(String typeName, int index){
		
		if(objectTypes.containsKey(typeName)){
			Vector objectTypeTable = (Vector) objectTypes.get(typeName);
			return (Element) objectTypeTable.elementAt(index);
			
		}
		return null;
	}
	//---------------------------------------------------------------------------
	/**
	 * Metoden getObjects henter en liste med alle objektene av en bestemt
	 * ObjectType.
	 * 
	 * @param string
	 * @return
	 */
	public Element[] getObjects(String typeName){
		
		if(objectTypes.containsKey(typeName)){
			Vector objectTypeTable = (Vector) objectTypes.get(typeName);
			Element[] retur = new Element[objectTypeTable.size()];
			
			for(int i = 0; i < objectTypeTable.size(); i++){
				retur[i] = (Element) objectTypeTable.elementAt(i);
			}
			return retur;
		}
		
		return null;
	}
	//------------------------------------------------------------------------------
	/**
	 * Metoden splitter ObjectTypene og lagrer.
	 * 
	 * @param target 
	 * @param objectTypes
	 */
	public void addObjectTypesUnParsed(Element target, String objectTypes){
		String[] per = objectTypes.split(" ");
		for (int i = 0; i < per.length; i++){
			addObjectType(target, per[i]);
		}
	}
	//--------------------------------------------------------------------------------
	/**
	 * Metoden sjekker om tabellen objectTypes inneholder funnet objectType. 
	 * Hvis den gjr det henter den objectType, hvis ikke legger den til 
	 * objectType i vektoren objectTypeTable.
	 * 
	 * @param target
	 * @param type
	 */
	public void addObjectType(Element target, String type){
		
		if(objectTypes.containsKey(type)){
			Vector objectTypeTable = (Vector) objectTypes.get(type);
			objectTypeTable.addElement(target);
			
		}
		else{
			Vector objectTypeTable = new Vector();
			objectTypeTable.addElement(target);
			objectTypes.put(type, objectTypeTable);
		}
	}
	
}
